{ **************************************************************************** }
{ *                                                                          * }
{ * LAS VEGAS (Bandai, LCD Game Digital) Simulator                           * }
{ * by Luca "MADrigal" Antignano, December 1999 (revision: August 2017)      * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit gfxinit;
{ this unit contains the graphics initialization (skin-based system)           }

interface

uses
  Forms, Controls, StdCtrls, SysUtils, Classes, Graphics, ExtCtrls, Dialogs,
  Jpeg, Pngimage, IniFiles;

{ * PROCEDURES: SKIN INITIALIZATION                                          * }
procedure Gfx_Game_Init;
procedure Gfx_Manual_Init;

const
  INIFILE = 'Config.ini';

{}IMAGES = 93;

{}DEFAULTHEIGHT    = 312;
{}DEFAULTWIDTH     = 546;
  DEFAULTSKINDIR   = 'Graphics\Skin\';
  DEFAULTWINMODE   = 'Fullscreen';

  DEFAULTTRCOLOR   = '$00ff00ff';

  DEFAULTBDDIR     = 'Graphics\Backdrops\';
  DEFAULTBDFILE    = 'nil';
  DEFAULTBDSTRETCH = 0;
{}DEFAULTBGCOLOR   = '$00000000';
  DEFAULTOFFSETX   = 'Center';
  DEFAULTOFFSETY   = 'Center';
  DEFAULTLOGOFILE  = 'nil';
  DEFAULTLOGOPOS   = 1;

  DEFAULTMANUALDIR = 'Graphics\Manual\';
{}DEFAULTMANHEIGHT = 950;
{}DEFAULTMANWIDTH  = 740;

var
  IF_File: TIniFile;
  aImages: array [1..IMAGES] of TImage;
{}//sVideoMode: string;

implementation

uses
  manual, unit1;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: SKIN INITIALIZATION                                          * }
{ *                                                                          * }
{ **************************************************************************** }

procedure Gfx_Game_Init;
{ initializes the graphics on the main game form                               }
  var
    // service variables
    i,
    iOffsetX,
    iOffsetY: integer;

    // temporary variables (used to read/write from 'ini file')
    sSkinDir,
    sBackdropDir,
    sBackdropFile,
    sGameOffsetX,
    sGameOffsetY,
{}  sLogoFile: string;
{}//sWindowMode

    // image variables
    sFilename: string;
begin
  // initialises array of images
  with Form1 do
    begin
      aImages[1]  := im_background;   aImages[2]  := im_background_screen_top;
      aImages[3]  := btn_mode_down;   aImages[4]  := btn_mode_top;
      aImages[5]  := btn_sound_off;   aImages[6]  := btn_sound_top;
      aImages[7]  := btn_acl_down;    aImages[8]  := btn_acl_top;

      aImages[9]  := btn_left_down;   aImages[10] := btn_left_top;
      aImages[11] := btn_right_down;  aImages[12] := btn_right_top;

      aImages[13] := im_panel_time; aImages[14] := im_panel_game_1; aImages[15] := im_panel_game_2;

      aImages[16] := im_panel_digit_pos_1; aImages[17] := im_panel_digit_pos_2; aImages[18] := im_panel_digit_pos_3; aImages[19] := im_panel_digit_pos_4; aImages[20] := im_panel_digit_pos_5;
      aImages[21] := im_panel_time_semicolon;
      aImages[22] := im_panel_miss_1; aImages[23] := im_panel_miss_2; aImages[24] := im_panel_miss_3;

      aImages[25] := im_panel_number_1_1;
      aImages[26] := im_panel_number_2_0; aImages[27] := im_panel_number_2_1; aImages[28] := im_panel_number_2_2; aImages[29] := im_panel_number_2_3; aImages[30] := im_panel_number_2_4; aImages[31] := im_panel_number_2_5; aImages[32] := im_panel_number_2_6; aImages[33] := im_panel_number_2_7; aImages[34] := im_panel_number_2_8; aImages[35] := im_panel_number_2_9;
      aImages[36] := im_panel_number_3_0; aImages[37] := im_panel_number_3_1; aImages[38] := im_panel_number_3_2; aImages[39] := im_panel_number_3_3; aImages[40] := im_panel_number_3_4; aImages[41] := im_panel_number_3_5; aImages[42] := im_panel_number_3_6; aImages[43] := im_panel_number_3_7; aImages[44] := im_panel_number_3_8; aImages[45] := im_panel_number_3_9;
      aImages[46] := im_panel_number_4_0; aImages[47] := im_panel_number_4_1; aImages[48] := im_panel_number_4_2; aImages[49] := im_panel_number_4_3; aImages[50] := im_panel_number_4_4; aImages[51] := im_panel_number_4_5; aImages[52] := im_panel_number_4_6; aImages[53] := im_panel_number_4_7; aImages[54] := im_panel_number_4_8; aImages[55] := im_panel_number_4_9;
      aImages[56] := im_panel_number_5_0;

      aImages[57] := im_hero_1; aImages[58] := im_hero_2; aImages[59] := im_hero_3;

      aImages[60] := im_player_1a; aImages[61] := im_player_2a; aImages[62] := im_player_3a;
      aImages[63] := im_player_1b; aImages[64] := im_player_2b; aImages[65] := im_player_3b;

      aImages[66] := im_slot_1; aImages[67] := im_slot_2; aImages[68] := im_slot_3;

      aImages[69] := im_coin_1_1; aImages[70] := im_coin_2_1; aImages[71] := im_coin_3_1;
      aImages[72] := im_coin_1_2; aImages[73] := im_coin_2_2; aImages[74] := im_coin_3_2;
      aImages[75] := im_coin_1_3; aImages[76] := im_coin_2_3; aImages[77] := im_coin_3_3;
      aImages[78] := im_coin_1_4; aImages[79] := im_coin_2_4; aImages[80] := im_coin_3_4;

      aImages[81] := im_slot_777_pos_1; aImages[82] := im_slot_777_pos_2; aImages[83] := im_slot_777_pos_3;
      aImages[84] := im_slot_777_number_1; aImages[85] := im_slot_777_number_3; aImages[86] := im_slot_777_number_7;
      aImages[87] := im_slot_777_hero_1; aImages[88] := im_slot_777_hero_3; aImages[89] := im_slot_777_hero_2;
      aImages[90] := im_slot_777_bunny;
      aImages[91] := im_slot_777_bag_1; aImages[92] := im_slot_777_bag_2; aImages[93] := im_slot_777_bag_3;
    end;

  // TO BE USED TO GENERATE THE SKIN FILE ONLY (TO BE COMMENTED THEREAFTER)
{  IF_File := TIniFile.Create(ExtractFilePath(ParamStr(0)) + 'Export.ini');

  for i := 1 to IMAGES do
    begin
      if aImages[i].Picture.Graphic = nil then
        sFilename := 'nil'
      else
        if aImages[i].Picture.Graphic is TJPEGImage then
          sFilename := aImages[i].Name + '.jpg'
        else
          if aImages[i].Picture.Graphic is TBitmap then
            sFilename := aImages[i].Name + '.bmp'
          else
            if aImages[i].Picture.Graphic is TPNGObject then
              sFilename := aImages[i].Name + '.png';

      IF_file.WriteString('Image' + IntToStr(i),  'Filename', sFilename);
      IF_file.WriteInteger('Image' + IntToStr(i), 'Height',   aImages[i].Height);
      IF_file.WriteInteger('Image' + IntToStr(i), 'Left',     aImages[i].Left);
      IF_file.WriteInteger('Image' + IntToStr(i), 'Top',      aImages[i].Top);
      IF_file.WriteInteger('Image' + IntToStr(i), 'Width',    aImages[i].Width);
    end;

  IF_File.Destroy; }

  // loads configuration settings from the Config.ini file
  if not FileExists(ExtractFilePath(ParamStr(0)) + INIFILE) then
    begin
      MessageDlg('Configuration file (Config.ini) not found. Please reinstall the game.', mtError, [mbOK], 0);
      Application.Terminate;
    end
  else
    begin
      // reads init values from 'ini file' -- with defaults if no 'ini file' found
      IF_File := TIniFile.Create(ExtractFilePath(ParamStr(0)) + INIFILE);

      // game form initialization (game graphics and backdrop)
      with Form1 do
        begin
          sSkinDir     := IncludeTrailingPathDelimiter(IF_file.ReadString('Common', 'SkinDir', DEFAULTSKINDIR));
          ClientHeight := IF_file.ReadInteger('Common', 'GameHeight', DEFAULTHEIGHT);
          ClientWidth  := IF_file.ReadInteger('Common', 'GameWidth',  DEFAULTWIDTH);

          // use Fullscreen or Shaped mode
{}        if bFullscreenMode then
            begin                                                               // Fullscreen
              // sets backdrop image and background colour
              sBackdropDir  := IncludeTrailingPathDelimiter(IF_file.ReadString('Fullscreen', 'BackdropDir',  DEFAULTBDDIR));
              sBackdropFile := IF_file.ReadString('Fullscreen', 'BackdropFile', DEFAULTBDFILE);

              if (sBackdropFile <> '')
                and (sBackdropFile <> 'nil') then
                if FileExists(sBackdropDir + sBackdropFile) then
                  begin
                    im_backdrop.Picture.LoadFromFile(sBackdropDir + sBackdropFile);
                    im_backdrop.Stretch := Boolean(IF_file.ReadInteger('Fullscreen', 'BackdropStretch', DEFAULTBDSTRETCH));
{}                  im_backdrop.Visible := true;
                  end;

              Color := StrToInt(IF_file.ReadString('Fullscreen', 'BgColor', DEFAULTBGCOLOR));
              TransparentColor := false;

              // sets new coordinates for the handheld game graphics
              sGameOffsetX := IF_file.ReadString('Fullscreen', 'GameOffsetX', DEFAULTOFFSETX);
              sGameOffsetY := IF_file.ReadString('Fullscreen', 'GameOffsetY', DEFAULTOFFSETY);

              if StrToIntDef(sGameOffsetX, 0) <> 0 then iOffsetX := StrToInt(sGameOffsetX) else iOffsetX := Trunc((Screen.Width  - ClientWidth)  / 2);
              if StrToIntDef(sGameOffsetY, 0) <> 0 then iOffsetY := StrToInt(sGameOffsetY) else iOffsetY := Trunc((Screen.Height - ClientHeight) / 2);

              // sets the form as full screen
              ClientWidth  := Screen.Width;
              ClientHeight := Screen.Height;

              // sets logo (watermark) image and its position
              sLogoFile := IF_file.ReadString('Fullscreen', 'LogoFile', DEFAULTLOGOFILE);

              if (sLogoFile <> '')
                and (sLogoFile <> 'nil') then
                if FileExists(sBackdropDir + sLogoFile) then
                  with Form1 do
                    begin
                      im_backdrop_logo.Picture.LoadFromFile(sBackdropDir + sLogoFile);
                      im_backdrop_logo.AutoSize := true;
{}                    im_backdrop_logo.Visible := true;

                      case IF_file.ReadInteger('Fullscreen', 'LogoPosition', DEFAULTLOGOPOS) of
                        0: begin im_backdrop_logo.Left := 0;                                    im_backdrop_logo.Top := 0; end; // top-left
                        1: begin im_backdrop_logo.Left := ClientWidth - im_backdrop_logo.Width; im_backdrop_logo.Top := 0; end; // top-right
                        2: begin im_backdrop_logo.Left := 0;                                    im_backdrop_logo.Top := ClientHeight - im_backdrop_logo.Height; end; // bottom-left
                        3: begin im_backdrop_logo.Left := ClientWidth - im_backdrop_logo.Width; im_backdrop_logo.Top := ClientHeight - im_backdrop_logo.Height; end; // bottom-right
                      end; // case..of
                    end;
            end
          else                                                                  // Shaped
            begin
              // sets coordinates for the handheld game graphics
              iOffsetX := 0;
              iOffsetY := 0;

              // sets the form as 'shaped'
              Color                 := StrToInt(IF_file.ReadString('VideoMode', 'TranspColor', DEFAULTTRCOLOR));
              TransparentColorValue := StrToInt(IF_file.ReadString('VideoMode', 'TranspColor', DEFAULTTRCOLOR));
              TransparentColor      := true;

              // disables the backdrop
              im_backdrop.Visible := false;
              im_backdrop_logo.Visible := false;
            end;
        end;

      // game skin initialization
      for i := 1 to IMAGES do
        begin
          sFilename := IF_file.ReadString('Image' + IntToStr(i), 'Filename', '');

          if (sFilename <> '')
            and (sFilename <> 'nil') then
            if FileExists(sSkinDir + sFilename) then
              // loads picture into the image
              aImages[i].Picture.LoadFromFile(sSkinDir + sFilename)
            else
              // image expected but not found -> creates a coloured square as a placeholder
              begin
                aImages[i].Picture.Bitmap.Create;
                aImages[i].Picture.Bitmap.Width  := 10;
                aImages[i].Picture.Bitmap.Height := 10;
                aImages[i].Picture.Bitmap.Canvas.Create;
                aImages[i].Picture.Bitmap.Canvas.Pen.Color := 0;
                aImages[i].Picture.Bitmap.Canvas.Pen.Style := psSolid;
                aImages[i].Picture.Bitmap.Canvas.Brush.Color := $000000ff;
                aImages[i].Picture.Bitmap.Canvas.Rectangle(0, 0, 10, 10);
              end;

          aImages[i].Height := IF_file.ReadInteger('Image' + IntToStr(i), 'Height', 0);
          aImages[i].Left   := IF_file.ReadInteger('Image' + IntToStr(i), 'Left',   0) + iOffsetX;
          aImages[i].Top    := IF_file.ReadInteger('Image' + IntToStr(i), 'Top',    0) + iOffsetY;
          aImages[i].Width  := IF_file.ReadInteger('Image' + IntToStr(i), 'Width',  0);
        end;

      // close Inifile
      IF_File.Destroy;
    end;

  // initializes the on-screen game menu
  with Form1 do
    begin
{}    btn_game_menu_top.Visible := bFullscreenMode;
{}    btn_game_menu_background.Visible := bFullscreenMode;

      btn_game_menu_background.Height := 32;
      btn_game_menu_background.Width := 36;
    end;
end;





procedure Gfx_Manual_Init;
{ initializes the graphics of the instructions manual form                     }
  var
    // temporary variables (used to read/write from 'ini file')
    sManualDir: string;

    // image variables
    sFilename: string;
begin
  // reads init values from 'ini file' -- with defaults if no 'ini file' found
  IF_File := TIniFile.Create(ExtractFilePath(ParamStr(0)) + INIFILE);

  // manual form initialization (graphics and sizes)
  with ManualForm do
    begin
      sManualDir   := IncludeTrailingPathDelimiter(IF_file.ReadString('Manual', 'ManualDir', DEFAULTMANUALDIR));
      sFilename := '1.bmp';

      ClientHeight := 400; // set size (to enable the 4 arrow buttons)
      ClientWidth  := 480; // set size (to enable the 4 arrow buttons)

      im_page.Left   := 0;
      im_page.Top    := 0;
      im_page.Height := IF_file.ReadInteger('Manual', 'ManualHeight', DEFAULTMANHEIGHT);
      im_page.Width  := IF_file.ReadInteger('Manual', 'ManualWidth',  DEFAULTMANWIDTH);

      sh_page_top.Left   := 0;
      sh_page_top.Top    := 0;
      sh_page_top.Height := ClientHeight;
      sh_page_top.Width  := ClientWidth;

      if FileExists(sManualDir + sFilename) then
        // loads picture into the image
        im_page.Picture.LoadFromFile(sManualDir + sFilename)
      else
        // image expected but not found -> creates a coloured square as a placeholder
        begin
          im_page.Picture.Bitmap.Create;
          im_page.Picture.Bitmap.Width  := 10;
          im_page.Picture.Bitmap.Height := 10;
          im_page.Picture.Bitmap.Canvas.Create;
          im_page.Picture.Bitmap.Canvas.Pen.Color := 0;
          im_page.Picture.Bitmap.Canvas.Pen.Style := psSolid;
          im_page.Picture.Bitmap.Canvas.Brush.Color := $000000ff;
          im_page.Picture.Bitmap.Canvas.Rectangle(0, 0, 10, 10);
        end;
    end;

  // close Inifile
  IF_File.Destroy;
end;





end.
